<?php


use Tests\TestCase;

require_once __DIR__.'/../_stub/Page/Abstract.php';
require_once __DIR__.'/../../../CoreExt/_stub/EntityDataMapper.php';

class Core_Controller_Page_AbstractTest extends TestCase
{
  protected $_pageController = NULL;

  protected function setUp()
  {
      parent::setUp();

    $this->_pageController = new Core_Controller_Page_AbstractStub();
  }

  /**
   * @expectedException CoreExt_Exception_InvalidArgumentException
   */
  public function testOpcaoDeConfiguracaoNaoExistenteLancaExcecao()
  {
    $this->_pageController->setOptions(array('foo' => 'bar'));
  }

  public function testClasseDataMapperEGeradaAPartirDaDefinicaoString()
  {
    $this->_pageController->_dataMapper = 'CoreExt_EntityDataMapperStub';
    $this->assertInstanceOf('CoreExt_DataMapper', $this->_pageController->getDataMapper(), 'Falhou na asserção a partir da instanciação de "Core_Page_Controller_Abstract".');

    $this->_pageController->setOptions(array('datamapper' => 'CoreExt_EntityDataMapperStub'));
    $this->assertInstanceOf('CoreExt_DataMapper', $this->_pageController->getDataMapper(), 'Falhou na asserção a partir de configuração via "setOptions()".');

    $this->_pageController->setDataMapper('CoreExt_EntityDataMapperStub');
    $this->assertInstanceOf('CoreExt_DataMapper', $this->_pageController->getDataMapper(), 'Falhou na asserção a partir de configuração via "setDataMapper()" com nome de classe "CoreExt_DataMapper".');

    $this->_pageController->setDataMapper(new CoreExt_EntityDataMapperStub());
    $this->assertInstanceOf('CoreExt_DataMapper', $this->_pageController->getDataMapper(), 'Falhou na asserção a partir de configuração via "setDataMapper()" com objeto "CoreExt_DataMapper".');
  }

  /**
   * @expectedException Core_Controller_Page_Exception
   */
  public function testClasseDataMapperNaoExistenteLancaExcecao()
  {
    $this->_pageController->setDataMapper('FooDataMapper');
  }

  /**
   * @expectedException CoreExt_Exception_InvalidArgumentException
   */
  public function testMetodoLancaExcecaoQuandoNaoRecebeTipoSuportado()
  {
    $this->_pageController->setDataMapper(0);
  }

  /**
   * @expectedException Core_Controller_Page_Exception
   */
  public function testClasseDataMapperNaoInformadaEMetodoNaoSubclassificadoLancaExcecao()
  {
    $this->_pageController->getDataMapper();
  }

  public function testAtribuicaoDeInstanciaEntity()
  {
    $this->_pageController->setEntity(new CoreExt_EntityStub());
    $this->assertInstanceOf('CoreExt_Entity', $this->_pageController->getEntity());
  }

  public function testInstanciaUmEntityCasoNenhumaInstanciaTenhaSidoAtribuidaExplicitamente()
  {
    $this->_pageController->setDataMapper('CoreExt_EntityDataMapperStub');
    $this->assertInstanceOf('CoreExt_Entity', $this->_pageController->getEntity());
  }

  public function testNumeroDoProcessoConfigurado()
  {
    $this->_pageController->_processoAp = 1;
    $this->assertInternalType('int', $this->_pageController->getBaseProcessoAp(), 'Falhou na asserção por tipo a partir da instanciação de "Core_Page_Controller_Abstract".');
    $this->assertEquals(1, $this->_pageController->getBaseProcessoAp(), 'Falhou na asserção por valor a partir da instanciação de "Core_Page_Controller_Abstract".');

    $this->_pageController->setOptions(array('processoAp' => 2));
    $this->assertEquals(2, $this->_pageController->getBaseProcessoAp(), 'Falhou na asserção a partir de configuração via "setOptions()".');

    $this->_pageController->setBaseProcessoAp(3);
    $this->assertEquals(3, $this->_pageController->getBaseProcessoAp(), 'Falhou na asserção a partir de configuração via "setBaseProcessoAp()".');
  }

  /**
   * @expectedException Core_Controller_Page_Exception
   */
  public function testNumeroDoProcessoNaoInformadoEMetodoNaoSubclassificadoLancaExcecao()
  {
    $this->_pageController->getBaseProcessoAp();
  }

  public function testTituloConfigurado()
  {
    $this->_pageController->_titulo = 'foo';
    $this->assertInternalType('string', $this->_pageController->getBaseTitulo(), 'Falhou na asserção por tipo a partir da instanciação de "Core_Page_Controller_Abstract".');
    $this->assertEquals('foo', $this->_pageController->getBaseTitulo(), 'Falhou na asserção por valor a partir da instanciação de "Core_Page_Controller_Abstract".');

    $this->_pageController->setOptions(array('titulo' => 'bar'));
    $this->assertEquals('bar', $this->_pageController->getBaseTitulo(), 'Falhou na asserção a partir de configuração via "setOptions()".');

    $this->_pageController->setBaseTitulo('zoo');
    $this->assertEquals('zoo', $this->_pageController->getBaseTitulo(), 'Falhou na asserção a partir de configuração via "setBaseTitulo()".');
  }

  /**
   * @expectedException Core_Controller_Page_Exception
   */
  public function testTituloNaoInformadoEMetodoNaoSubclassificadoLancaExcecao()
  {
    $this->_pageController->getBaseTitulo();
  }

  public function testAppendOutput()
  {
    $this->_pageController->appendOutput('string 1')
                          ->appendOutput('string 2');

    $this->assertEquals(
      'string 1' . PHP_EOL . 'string 2',
      $this->_pageController->getAppendedOutput(),
      '->getAppendedOutput() retorna o conteúdo a ser adicionado como uma string separada por quebra de linha'
    );
  }

  public function testGetApendedOutputRetornaNullQuandoNaoExisteConteudoASerAdicionado()
  {
    $this->assertNull($this->_pageController->getAppendedOutput());
  }

  public function testPrependOutput()
  {
    $this->_pageController->prependOutput('string 1')
                          ->prependOutput('string 2');

    $this->assertEquals(
      'string 1' . PHP_EOL . 'string 2',
      $this->_pageController->getPrependedOutput(),
      '->getPrependedOutput() retorna o conteúdo a ser adicionado como uma string separada por quebra de linha'
    );
  }

  public function testGetPrependedOutputRetornaNullQuandoNaoExisteConteudoASerAdicionado()
  {
    $this->assertNull($this->_pageController->getPrependedOutput());
  }
}
